# w.c.s. - web application for online forms
# Copyright (C) 2005-2016  Entr'ouvert
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

import sys
import xml.etree.ElementTree as ET

from quixote import get_publisher

from qommon import get_logger
from qommon.form import WidgetListAsTable, CompositeWidget, SingleSelectWidget, ComputedExpressionWidget
from wcs.workflows import XmlSerialisable, WorkflowStatusItem, register_item_class
from wcs.wf.profile import FieldNode


class SetBackofficeFieldRowWidget(CompositeWidget):
    def __init__(self, name, value=None, workflow=None, **kwargs):
        CompositeWidget.__init__(self, name, value, **kwargs)
        if not value:
            value = {}

        fields = [('', '', '')]
        fields.extend([(x.id, x.label, x.id) for x in workflow.get_backoffice_fields()])
        self.add(SingleSelectWidget, name='field_id', title=_('Field'),
                value=value.get('field_id'),
                options=fields, **kwargs)
        self.add(ComputedExpressionWidget, name='value', title=_('Value'),
                value=value.get('value'))

    def _parse(self, request):
        if self.get('value') and self.get('field_id'):
            self.value = {
                'value': self.get('value'),
                'field_id': self.get('field_id')
            }
        else:
            self.value = None


class SetBackofficeFieldsTableWidget(WidgetListAsTable):
    readonly = False
    def __init__(self, name, **kwargs):
        super(SetBackofficeFieldsTableWidget, self).__init__(name,
                element_type=SetBackofficeFieldRowWidget,
                element_kwargs={'workflow': kwargs.pop('workflow')},
                **kwargs)


class SetBackofficeFieldsWorkflowStatusItem(WorkflowStatusItem):
    description = N_('Set Backoffice Fields')
    key = 'set-backoffice-fields'

    fields = None

    @classmethod
    def is_available(cls, workflow=None):
        return bool(workflow and getattr(workflow.backoffice_fields_formdef, 'fields', None))

    def get_parameters(self):
        return ('fields',)

    def add_parameters_widgets(self, form, parameters, prefix='', formdef=None):
        if 'fields' in parameters:
            form.add(SetBackofficeFieldsTableWidget, '%sfields' % prefix,
                    title=_('Fields Update'), value=self.fields,
                    workflow=self.parent.parent)

    def perform(self, formdata):
        if not self.fields:
            return
        for field in self.fields:
            try:
                formdata.data['%s' % field['field_id']] = self.compute(
                        field['value'], raises=True)
            except:
                get_publisher().notify_of_exception(sys.exc_info())
        formdata.store()

    def fields_export_to_xml(self, item, charset, include_id=False):
        if not self.fields:
            return

        fields_node = ET.SubElement(item, 'fields')
        for field in self.fields:
            fields_node.append(FieldNode(field).export_to_xml(charset=charset,
                include_id=include_id))

        return fields_node

    def fields_init_with_xml(self, elem, charset, include_id=False):
        fields = []
        if elem is None:
            return
        for field_xml_node in elem.findall('field'):
            field_node = FieldNode()
            field_node.init_with_xml(field_xml_node, charset,
                    include_id=include_id)
            fields.append(field_node.as_dict())
        if fields:
            self.fields = fields

register_item_class(SetBackofficeFieldsWorkflowStatusItem)
